; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2017 Average Software.

; Main game routines

; Ice color value.
ice_color           equ     $d

; Rock color values.
rock_color_0        equ     $d6
rock_color_1        equ     $32

; Addresses for the rock to be shot.
throw_address_left  equ     $2cc5
throw_address_right equ     $288e

; End addresses for the weight meter.
weight_meter_left   equ     $235c
weight_meter_right  equ     weight_meter_left + 5

; Base address for the curl meter.
curl_meter_base     equ     $27ab
; Meter center address, all drawing starts from there.
curl_meter_center   equ     $2b3c

; Base addresses for the status rocks.
p0_status_base      equ     $3750
p1_status_base      equ     $3768

; Color values for the rings.
outer_ring_color    equ     $80
inner_ring_color    equ     $32

; Base player location for the rings.
ring_base           equ     175

; Hog line missile location.
front_hog_line      equ     ring_base - 37

; Back line missile location.
back_line           equ     ring_base + 30

; End-cap line for the scoreboard location.
end_cap_line        equ     208

; Bytes per sheet row.
bytes_per_row       equ     83

; Distance to the front hogline.
hogline_distance    equ     70

; Delay time between shots.
shot_end_delay_time equ     120

; Rock data fields.
rock_address        equ     0
rock_offset         equ     2
rock_handle         equ     3
rock_h_speed        equ     4
rock_v_speed        equ     5
rock_r_speed        equ     6
rock_h_time         equ     7
rock_v_time         equ     8
rock_r_time         equ     9
rock_flags          equ     10
rock_data_size      equ     11

; Bit fields for the rock flags.
rock_team           equ     1
rock_rotation       equ     2
rock_mode           equ     4

; Shot mode constants.
shot_mode_none      equ     0
shot_mode_aim       equ     1
shot_mode_weight    equ     2
shot_mode_curl      equ     3

; Joystick bounds constants.
aim_lower_bound     equ     68
aim_upper_bound     equ     133
weight_lower_bound  equ     83
weight_upper_bound  equ     115
curl_lower_bound    equ     87
curl_upper_bound    equ     111
curl_range          equ     curl_upper_bound - curl_lower_bound

; Trigger debounce length.
trigger_debounce    equ     10

; Status blink timer length.
blink_length        equ     30

; Rock data storage.
rock_data_1         equ     $19
; List of rocks that need processing.
rock_list           equ     $d0
; Aim cursor position, two byte address.
; $2031 is the upper boundary, $3544 the lower.
; $2a91 is the center.
aim_cursor          equ     $e0
; Current shot mode.
shot_mode           equ     $e2
; Last joystick position.
last_stick          equ     $e3
; Trigger buffer for preventing repeats.
trigger_buffer      equ     $e4
; De-bounce timer for the trigger.
trigger_timer       equ     $e5
; The address of the rock currently being thrown.
current_rock        equ     $e6
; The blink timer for the rock display at the bottom.
blink_timer         equ     $e7
; The current scroll value.
current_scroll      equ     $e8
; The pointer offset into the scroll area of the next display list.
scroll_list_offset  equ     $e9
; Lower sheet colors used in interrupts.
pm1_low_color_2     equ     $ea
pm2_low_color_2     equ     $eb
pm1_low_color_3     equ     $ec
pm2_low_color_3     equ     $ed

; Player/missile positions for the bottom sheet.
p0_low_h            equ     $300
p1_low_h            equ     $301
p2_low_h            equ     $302
p3_low_h            equ     $303
m0_low_h            equ     $304
m1_low_h            equ     $305
m2_low_h            equ     $306
m3_low_h            equ     $307
; Player missile colors for the bottom sheet.
; PM0 is always blue, so no need for that.
; PM3 only has one color.
pm1_low_color_1     equ     $308
pm2_low_color_1     equ     $309
pm3_low_color       equ     $30a
; Counters for ice friction on the rocks.
rock_friction       equ     $310
; 16-bit counter for curl application on the shooter.
rock_curl           equ     $320
; Horizontal tracking for each rock.
rock_progress       equ     $322
rock_progress_high  equ     rock_progress + 16
; Delay counter for the between shot pause.
shot_end_delay      equ     rock_progress_high + 16

; Set the initial game parameters and kick off the game loop.
; The ice sheet lines start at $201d so that the 4k boundary ($3000)
; will align perfectly with the left edge of the sheet.
start_game subroutine
    mva     #$70,$400               ; Start building the display list.
    sta     $401                    ; 24 blank scanlines for the overscan.
    sta     $402
    mva     #$c2,$403               ; A line of mode 2, with an interrupt.
    mwa     $1000,$404              ; Start screen RAM at $1000.
    mwa     [$201d + bytes_per_row - 40],$fe
    ldy     #0                      ; 69 lines of mode e.
.top_area_loop
    lda     #$4e                    ; Load memory on every line.
    sta     $406,y
    iny
    lda     $fe                     ; Write the memory address.
    sta     $406,y
    iny
    lda     $ff
    sta     $406,y
    clc                             ; Update the address.
    lda     $fe
    adc     #bytes_per_row
    bcc     .no_address_carry
    inc     $ff
.no_address_carry
    sta     $fe
    iny
    cpy     #69 * 3
    bne     .top_area_loop          ; Quit after 69 lines.
    mva     #$ce,$4d5               ; One more e, with an interrupt.
    sta     $406 + 24 * 3           ; Set two more interrupts for
    sta     $406 + 44 * 3           ; the ring colors.
    sta     $406 + 33 * 3           ; Set two interrupts for the
    sta     $406 + 35 * 3           ; center line.
    mva     $fe,$4d6                ; Set the address for the last
    mva     $ff,$4d7                ; ice sheet line.
    mva     #$42,$4d8               ; Top line of the score sheet.
    mwa     $1028,$4d9
    mva     #$02,$4dc               ; Two more text lines with e in between.
    mva     #$82,$4de               ; Interrupt here to position the lower P/Ms.
    mva     #$0e,$4db
    sta     $4dd
    mva     #$8e,$4df               ; Interrupt on the last score line.
    mwa     $201d,$fe               ; Start the bottom ice sheet on the far left.
    ldy     #0                      ; 69 more rows.
.bottom_area_loop
    lda     #$5e                    ; Load memory and hscroll on every line.
    sta     $4e0,y
    iny
    lda     $fe                     ; Write the memory address.
    sta     $4e0,y
    iny
    lda     $ff
    sta     $4e0,y
    clc                             ; Update the address.
    lda     $fe
    adc     #bytes_per_row
    bcc     .no_address_carry2
    inc     $ff
.no_address_carry2
    sta     $fe
    iny
    cpy     #69 * 3
    bne     .bottom_area_loop       ; Quit after 69 lines.
    mva     #$de,$5af               ; One more e, with an interrupt and scroll.
    sta     $4e0 + 23 * 3           ; Set four more interrupts for
    sta     $4e0 + 45 * 3           ; the ring colors.
    sta     $4e0 + 24 * 3           ; These need more interrupts because the
    sta     $4e0 + 46 * 3           ; scroll processing eats some cycles.
    sta     $4e0 + 33 * 3           ; Set two interrupts for the
    sta     $4e0 + 35 * 3           ; center line.
    mva     $fe,$5b0                ; Set the address for the last
    mva     $ff,$5b1                ; ice sheet line.
    mva     #$4e,$5b2               ; Set the beginning of the bottom area.
    mwa     $3700,$5b3              ; Start at $3700.
    ldy     #15                     ; 16 more rows of e.
    lda     #$0e
.status_area_loop
    sta     $5b5,y
    dey
    bpl     .status_area_loop
    mva     #$8e,$5b8               ; Set interrupts for status coloring.
    sta     $5ba
    mva     #$41,$5c5               ; Jump instruction.
    mwa     $400,$5c6               ; Go back to $400.
    ldy     #0                      ; Make another copy of the display list.
.copy_dlist_loop
    lda     $400,y                  ; This is used for scrolling performance.
    sta     $800,y
    lda     $500,y
    sta     $900,y
    iny
    bne     .copy_dlist_loop
    mwa     $800,$9c6               ; Change the copy's reset address.
    clc                             ; Build the scoreboard.
    lda     #$11                    ; Start with '1'.
    ldy     #0
.end_number_loop
    sta     $1029,y
    adc     #1
    iny
    iny
    iny
    iny
    cmp     #$1a
    bne     .end_number_loop
    lda     #$11                    ; Put the 10 on the last column.
    sta     $1029,y
    iny
    lda     #$10
    sta     $1029,y
    lda     #124                    ; Put the | separators in.
    ldy     #0
.separator_loop
    sta     $1078,y
    sta     $10c8,y
    iny
    iny
    iny
    iny
    cpy     #40
    bne     .separator_loop
    lda     #$aa                    ; Now draw the horizontal lines.
    ldy     #39
.line_loop
    sta     $1050,y
    sta     $10a0,y
    sta     $10f0,y
    dey
    bne     .line_loop
    lda     #$0a                    ; These are the two pixels on the left.
    sta     $1050
    sta     $10a0
    sta     $10f0
    ldx     #3                      ; Now build the hacks, 3 pixels tall.
    ldy     #0                      ; Index of 0.
    mwa     [$201d + bytes_per_row * 31],$fe
    mwa     [$201d + bytes_per_row * 40],$fc
.hack_loop
    lda     #$fc                    ; Build the hack in black.
    sta     ($fe),y                 ; Write to the top hack.
    sta     ($fc),y                 ; Write to the bottom hack.
    dex
    beq     .done_hack              ; Quit after 3 rows.
    lda     $fe                     ; Move to the next row for both hacks.
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow_hack1
    dec     $ff
.no_borrow_hack1
    sta     $fe
    lda     $fc
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow_hack2
    dec     $fd
.no_borrow_hack2
    sta     $fc
    jmp     .hack_loop              ; Do it again.
.done_hack
    mwa     $a000,VDSLST            ; Set the initial interrupt.
    mwa     game_vblank,VVBLKD      ; Set the vblank interrupt.
    mva     #0,sCOLBK               ; Start the background color at black.
    sta     sCOLOR2                 ; Black text background.
    mva     #$0f,sCOLOR1            ; Full text luminance.
    mva     #rock_color_0,sCOLOR0   ; Team 0's rock color.
    mva     #outer_ring_color,sCOLPM0
    sta     sCOLPM1                 ; Set the players to blue for the outer ring.
    sta     sCOLPM2
    sta     sCOLPM3
    sta     pm1_low_color_1
    sta     pm2_low_color_1
    sta     pm3_low_color
    ldy     #$2d                    ; Build the ring pattern for the players,
    jsr     build_player_rings      ; at offset $2d.
    ldy     #$28                    ; Build the lines at offset $28.
    lda     #$82                    ; Use one pixel each from missiles 0 and 3.
    jsr     build_lines
    ldy     #$89                    ; Build the lines at $89 on the lower sheet.
    lda     #$82                    ; Use one pixel each from missiles 0 and 3.
    jsr     build_lines
    ldy     #$76                    ; Put the end-line on the scoreboard at $76.
    lda     #$04                    ; Use one pixel from missile 1.
    ldx     #19                     ; Write 19 bytes.
.score_cap_loop
    sta     $1300,y
    iny
    dex
    bne     .score_cap_loop
    mva     #$0c,$1376              ; Make the top, center, and bottom of the
    sta     $1388                   ; end-line two pixels wide to extend the
    sta     $137f                   ; horizontal lines.
    mva     #$55,$fd                ; Draw the shot rocks at the bottom
    mwa     p0_status_base,$fe      ; of the screen.
    jsr     draw_8_status_rocks
    mva     #$aa,$fd
    mwa     p1_status_base,$fe
    jsr     draw_8_status_rocks
    jsr     prepare_next_shot       ; Prepare the first rock.
    mva     #$c0,NMIEN              ; Set the interrupt flags.
    cli                             ; Enable interrupts.
    ; Fall through to the main game loop.

; Main game loop.
game_loop subroutine
    jsr     wait_for_vb
    lda     shot_mode               ; Check the shot mode.
    beq     .no_aim                 ; On 0, the shot is already in motion.
    jsr     handle_aim              ; Go do the aiming logic.
.no_aim
    ldy     #0                      ; Index into the moving rock list.
.move_rocks_loop
    ldx     rock_list,y             ; Load the next rock to be moved.
    beq     .done_move_rocks        ; The list is terminated by 0.
    tya                             ; Save y, it will get clobbered
    pha                             ; during this process.
    jsr     process_rock            ; Go handle rock movement and interactions.
    pla
    tay
    cpy     #15                     ; After 16 rocks, stop.
    beq     .done_move_rocks
    iny
    jmp     .move_rocks_loop
.done_move_rocks
    cpy     #0                      ; If y is still 0, and the shot mode
    bne     .check_blink            ; is shot_mode_none, the shot is complete.
    lda     shot_mode
    bne     .check_blink
    dec     shot_end_delay          ; Check the delay timer.
    bne     .check_blink
    jsr     prepare_next_shot
.check_blink
    dec     blink_timer             ; Check the status area blinker.
    bne     .done_status_blink
    jsr     blink_status_rock
.done_status_blink
    ldx     current_rock            ; If the current rock has moved
    lda     rock_h_speed,x          ; the h_time will be equal to
    beq     .done_scroll            ; the h_speed, and not equal to 0.
    jsr     apply_curl              ; The rock is still moving, so apply curl.
    cmp     rock_h_time,x           ; Check for scrolling.
    bne     .done_scroll
    jsr     get_rock_number_from_x
    lda     rock_h_speed,x
    cmp     #0                      ; Update the rock position based on
    bpl     .move_right             ; the direction of movement.
    tya
    tax
    dec     rock_progress,x
    jmp     .done_scroll            ; No left scrolling.
.move_right
    tya
    tax
    inc     rock_progress,x
    bne     .no_carry
    inc     rock_progress_high,x
.no_carry
    jsr     perform_scroll
.done_scroll
    jmp     game_loop

; Calculates the current rock number from the active rock pointer.
; Result is placed in x.
get_rock_number subroutine
    lda     #rock_data_1            ; Start from the rock base.
    ldx     #0                      ; Assume rock 0.
    clc
.loop
    cmp     current_rock
    bne     .next_rock
    rts
.next_rock
    adc     #rock_data_size
    inx
    jmp     .loop

; Calculate the number of the rock whose address is in x.
; Places the number in y.
get_rock_number_from_x subroutine
    lda     #rock_data_1            ; Start from the rock base.
    stx     $ff                     ; Put the address somewhere handy.
    ldy     #0                      ; Assume rock 0.
    clc
.loop
    cmp     $ff
    bne     .next_rock
    rts
.next_rock
    adc     #rock_data_size
    iny
    jmp     .loop

; Handle the shot aiming by the player.  Expects the
; current shot mode to be in the accumulator.
handle_aim subroutine
    cmp     #shot_mode_aim          ; Check for the aim mode.
    bne     .no_aim
    jsr     erase_aim_cursor
    lda     p0vpot                  ; Figure out where to draw the new cursor.
    cmp     #aim_lower_bound        ; Clamp to the lower boundary.
    bcs     .aim_lower_bound_ok
    lda     #aim_lower_bound
    jmp     .adjust_aim
.aim_lower_bound_ok
    cmp     #aim_upper_bound        ; Clamp to the upper boundary.
    bcc     .adjust_aim
    lda     #aim_upper_bound
.adjust_aim
    pha                             ; Save the actual value for later.
    sec
    sbc     last_stick              ; Figure out which direction to go.
    beq     .no_aim_adjust
    bcc     .move_aim_up            ; New value is less, move up.
    tax                             ; Counter for the rows to move.
    lda     aim_cursor              ; Get the cursor address.
.aim_down_loop
    clc
    adc     #bytes_per_row
    bcc     .no_carry1
    inc     aim_cursor + 1
.no_carry1
    dex
    beq     .aim_adjust_done
    jmp     .aim_down_loop
.move_aim_up
    tax                             ; Counter for the rows to move.
    lda     aim_cursor              ; Get the cursor address.
.aim_up_loop
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow
    dec     aim_cursor + 1
.no_borrow
    inx
    bne     .aim_up_loop
.aim_adjust_done
    sta     aim_cursor
.no_aim_adjust
    pla
    sta     last_stick              ; Update the joystick position.
    ldx     current_rock
    sta     rock_v_speed,x          ; Save the value in the rock for later.
    mva     aim_cursor,$fe          ; Reset the cursor address.
    mva     aim_cursor + 1,$ff
    ldx     #4                      ; Draw the new cursor, 5 pixels high.
    ldy     #0
.aim_draw_loop
    lda     aim_arrow,x
    sta     ($fe),y
    lda     $fe
    clc
    adc     #bytes_per_row
    bcc     .no_carry2
    inc     $ff
.no_carry2
    sta     $fe
    dex
    bpl     .aim_draw_loop
    jmp     .aim_done
.no_aim
    cmp     #shot_mode_weight       ; See if the shooter is setting weight.
    beq     .set_weight
    jmp     .no_weight
.set_weight
    lda     p0hpot                  ; Get the stick position.
    cmp     #weight_lower_bound     ; Check the lower bound.
    bcs     .weight_lower_bound_ok
    lda     #weight_lower_bound
    jmp     .adjust_weight
.weight_lower_bound_ok
    cmp     #weight_upper_bound     ; Check the upper bound.
    bcc     .adjust_weight
    lda     #weight_upper_bound
.adjust_weight
    sec
    sbc     #weight_lower_bound     ; Subtract the lower bound and then
    lsr                             ; divide by two to get the weight.
    sta     $fb                     ; Save it for later.
    ldx     current_rock
    sta     rock_h_speed,x          ; Stash it the rock for later processing.
    mwa     [weight_meter_left + 1],$fc
    ldy     #0
.weight_meter_loop_outer
    ldx     #6                      ; Weight scale is 6 high.
    mva     $fc,$fe                 ; Prepare the address to draw from.
    mva     $fd,$ff
    lda     $fb                     ; Get the weight back.
    cmp     #4                      ; If it's 4 or more, draw a full byte.
    bpl     .draw_full_weight_byte
    cmp     #3                      ; 6 bits.
    bne     .not_3_weight
    lda     #$a8
    jmp     .draw_weight_byte
.not_3_weight
    cmp     #2
    bne     .not_2_weight
    lda     #$a0                    ; 4 bits.
    jmp     .draw_weight_byte
.not_2_weight
    cmp     #1
    bne     .blank_weight_byte
    lda     #$80                    ; 2 bits.
    jmp     .draw_weight_byte
.blank_weight_byte
    lda     #0                      ; 0 bits, blank section.
    jmp     .draw_weight_byte
.draw_full_weight_byte
    lda     #$aa
.draw_weight_byte
    sta     ($fe),y                 ; Write the byte.
    dex
    bne     .update_weight_address
    lda     $fc                     ; On the fourth byte, drawing is done.
    cmp     [<#weight_meter_left] + 4
    bne     .draw_weight_continue
    jmp     .aim_done
.draw_weight_continue
    lda     $fb                     ; Get the weight back.
    sec
    sbc     #4                      ; Drop it by four.
    sta     $fb                     ; Update the remaining weight.
    inc     $fc                     ; Increment the starting address.
    jmp     .weight_meter_loop_outer
.update_weight_address
    pha                             ; Save the bit pattern.
    lda     $fe                     ; Advance to the next row.
    clc
    adc     #bytes_per_row
    bcc     .no_carry_weight_draw
    inc     $ff
.no_carry_weight_draw
    sta     $fe
    pla                             ; Get the bit pattern back.
    jmp     .draw_weight_byte
.no_weight
    lda     p0vpot                  ; Get the stick position.
    cmp     #curl_lower_bound       ; Check the lower bound.
    bcs     .curl_lower_bound_ok
    lda     #curl_lower_bound
    jmp     .adjust_curl
.curl_lower_bound_ok
    cmp     #curl_upper_bound       ; Check the upper bound.
    bcc     .adjust_curl
    lda     #curl_upper_bound
.adjust_curl
    sec
    sbc     #curl_lower_bound       ; Subtract the lower bound to get the base curl.
    sbc     #curl_range / 2         ; Subtract half the range of values to center at 0.
    sta     $fb                     ; Save it for later.
    ldx     current_rock
    sta     rock_r_speed,x          ; Stash it in the rock.
    mva     #$03,$fd                ; Prepare the base curl gauge.
    jsr     reset_curl_gauge
    lda     $fb                     ; Get the value back.
    bmi     .curl_up                ; On negative, draw the curl gauge going up.
    tax
    mwa     curl_meter_center,$fe   ; Prepare the base address for drawing.
    ldy     #0
.curl_meter_loop_down
    lda     #$2b                    ; Right pixel black, then two colored.
    sta     ($fe),y
    dex
    bmi     .aim_done
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry_curl
    inc     $ff
.no_carry_curl
    sta     $fe
    jmp     .curl_meter_loop_down
.curl_up
    tax
    mwa     curl_meter_center,$fe   ; Prepare the base address for drawing.
    ldy     #0
.curl_meter_loop_up
    lda     #$2b                    ; Right pixel black, then two colored.
    sta     ($fe),y
    inx
    beq     .aim_done
    lda     $fe                     ; Update the address.
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow_curl
    dec     $ff
.no_borrow_curl
    sta     $fe
    jmp     .curl_meter_loop_up
.aim_done
    lda     trigger_timer           ; Check the trigger debounce.
    beq     .process_trigger
    dec     trigger_timer
    jmp     .no_trigger_change
.process_trigger
    lda     TRIG0                   ; Check the trigger for confirmation.
    cmp     trigger_buffer          ; Only honor the trigger if it changed.
    bne     .trigger_change
    jmp     .no_trigger_change
.trigger_change
    cmp     #0
    beq     .no_trigger_release
    jmp     .trigger_release        ; On 1, the button was released.
.no_trigger_release
    inc     shot_mode               ; Go to the next shot mode.
    pha                             ; Save the trigger state.
    mva     #trigger_debounce,trigger_timer
    lda     shot_mode               ; Prep for the next shot mode.
    cmp     #shot_mode_weight
    beq     .prepare_weight_mode
    cmp     #shot_mode_curl
    beq     .prepare_curl_mode
    jmp     .release_shot
.prepare_weight_mode
    ldy     #0                      ; Draw the weight scale, 6 high.
    ldx     #6
    mwa     weight_meter_left,$fe   ; Left edge of the scale.
    mwa     weight_meter_right,$fc  ; Right edge of the scale.
.weight_scale_loop
    lda     #$03                    ; Two pixels on the right edge.
    sta     ($fe),y
    lda     #$c0                    ; Two pixels on the left edge.
    sta     ($fc),y
    dex
    bne     .continue_weight
    jmp     .finish_prepare
.continue_weight
    lda     $fe                     ; Update the addresses.
    clc
    adc     #bytes_per_row
    bcc     .no_carry_weight_loop1
    inc     $ff
.no_carry_weight_loop1
    sta     $fe
    lda     $fc
    clc
    adc     #bytes_per_row
    bcc     .no_carry_weight_loop2
    inc     $fd
.no_carry_weight_loop2
    sta     $fc
    jmp     .weight_scale_loop
.prepare_curl_mode
    ldx     current_rock            ; Evaluate the aim and weight.
    lda     #17
    sec
    sbc     rock_h_speed,x          ; Subtract the adjusted position from 17.
    pha                             ; Save it, need to do some manipulations.
    and     #1                      ; An even speed has a quicker friction value
    bne     .odd_speed              ; for the first slowdown.
    lda     #120
    jmp     .store_friction
.odd_speed
    lda     #0
.store_friction
    pha                             ; Save the friction.
    jsr     get_rock_number_from_x  ; Get the rock number so the friction
    pla                             ; can be set.
    sta     rock_friction,y
    pla                             ; Get the speed value back.
    lsr                             ; Divide it by two.
    bne     .store_h_speed
    lda     #1                      ; If the speed was one, the shift kills it.
.store_h_speed
    sta     rock_h_speed,x          ; Speed is expressed as "frames per pixel."
    sta     rock_h_time,x           ; Valid weights are 1 (fast) - 8 (slow).
    lda     rock_v_speed,x          ; Get the aim value.
    sec
    sbc     #aim_lower_bound + 28   ; Normalize it, and less than 28 is shooting up.
    bmi     .shoot_up
    bne     .shoot_down
    tay
    jmp     .store_aim              ; Aim of 0, can't do division!
.shoot_down
    sta     $ff                     ; This is the dividend.
    lda     #hogline_distance       ; Divisor is the distance to the line.
    jsr     divide
    jmp     .store_aim
.shoot_up
    eor     #$ff                    ; Take the absolute value.
    clc
    adc     #1
    sta     $ff                     ; This is the dividend.
    lda     #hogline_distance       ; Divisor is the distance to the line.
    jsr     divide
    tya                             ; Convert back to negative.
    eor     #$ff
    clc
    adc     #1
    tay
.store_aim
    sty     rock_v_speed,x
    sty     rock_v_time,x
    lda     #0
    sta     rock_r_time,x
    jsr     activate_rock
    jsr     erase_aim_cursor
    ldx     #6                      ; Erase the weight scale.
    mwa     weight_meter_left,$fe
.erase_weight_scale_loop
    ldy     #0
    tya                             ; Write 0s.
.erase_weight_scale_loop_inner
    sta     ($fe),y
    iny
    cpy     #weight_meter_right - #weight_meter_left + 1
    bne     .erase_weight_scale_loop_inner
    dex
    bne     .continue_weight_erase
    jmp     .finish_prepare
.continue_weight_erase
    lda     $fe
    clc
    adc     #bytes_per_row
    bcc     .no_carry_erase_weight_loop
    inc     $ff
.no_carry_erase_weight_loop
    sta     $fe
    jmp     .erase_weight_scale_loop
.release_shot
    mva     #shot_mode_none,shot_mode
    ldx     current_rock
    lda     rock_r_speed,x          ; Get the curl speed.
    bmi     .convert_curl_rate
    tay                             ; Save the value.
    lda     rock_flags,x            ; Set the direction flag.
    ora     #rock_rotation
    sta     rock_flags,x
    tya                             ; Get the speed back.
    jmp     .finish_curl
.convert_curl_rate
    eor     #$ff
    clc
    adc     #1                      ; Convert the negative curl to positive.
.finish_curl
    sta     $ff                     ; The curl rate needs to be "inverted"
    lda     #curl_range / 2 + 1     ; since high numbers are slower.
    sec
    sbc     $ff
    asl                             ; Now shift a couple times to scale it up.
    asl
    sta     rock_r_speed,x
    sta     rock_r_time,x
    mva     #0,$fd                  ; Now erase the curl gauge.
    jsr     reset_curl_gauge
.finish_prepare
    pla                             ; Retrieve the trigger state.
.trigger_release
    sta     trigger_buffer          ; Update the trigger buffer.
.no_trigger_change
    rts

; Reset the playfield pointers in the two display lists.
reset_scroll subroutine
    mvy     #$0f,current_scroll     ; Reset the fine scroll.
    sty     HSCROL
    mvy     #0,scroll_list_offset   ; Reset the scroll pointer.
    ldx     #70                     ; 70 rows to update.
    mwa     $4e1,$fe                ; Address of the first memory scan.
    mwa     $8e1,$fc                ; Address of the second memory scan.
    mwa     $201d,$fa               ; Starting address for list 1.
    mwa     $2019,$f8               ; Starting address for list 2.
.loop
    lda     $fa
    sta     ($fe),y
    lda     $f8
    sta     ($fc),y
    iny
    lda     $fb
    sta     ($fe),y
    lda     $f9
    sta     ($fc),y
    dex
    beq     .done
    iny                             
    iny                             ; Hop over the mode byte.
    lda     $fa
    clc
    adc     #bytes_per_row
    bcc     .no_carry_list_1
    inc     $fb
.no_carry_list_1
    sta     $fa
    lda     $f8
    clc
    adc     #bytes_per_row
    bcc     .no_carry_list_2
    inc     $f9
.no_carry_list_2
    sta     $f8
    jmp     .loop
.done
    mva     #$4,sDLISTH             ; Go back to the first display list.
    sta     DLISTH
    rts

; Perform all scrolling logic, updating display pointers and
; swapping the display lists if necessary.  Expects the current
; rock number in x.
perform_scroll subroutine
    ldy     rock_progress,x
    cpy     #80                     ; Don't scroll before 80 pixels.
    beq     .left_bound_fail
    bcs     .left_bound_ok
.left_bound_fail
    lda     shot_mode               ; If the rock is still being
    beq     .shot_released          ; held, move the curler to the right.
    inc     p0_low_h
    inc     p1_low_h
    inc     p2_low_h
    inc     p3_low_h
.shot_released
    rts
.left_bound_ok
    cpy     #252                    ; Don't scroll after 252.
    bcc     .right_bound_ok
    rts
.right_bound_ok
    cpy     #202                    ; Don't scroll the backline too early.
    bcc     .no_shift_m3
    dec     m3_low_h
.no_shift_m3
    ldy     m0_low_h
    cpy     #40
    bcs     .no_shift_m0
    ldy     #$8e
    jsr     build_player_rings      ; Create the player rings on the bottom.
    mva     #255,p0_low_h           ; Position the rings offscreen to the right.
    mva     #7,p1_low_h
    mva     #15,p2_low_h
    mva     #23,p3_low_h
    mva     #outer_ring_color,pm1_low_color_1
    sta     pm2_low_color_1
    sta     pm3_low_color
    sta     pm1_low_color_3
    sta     pm2_low_color_3
    mva     #inner_ring_color,pm1_low_color_2
    sta     pm2_low_color_2
    ldy     #218                    ; Reposition m0 to the other hogline.
.no_shift_m0
    dey
    sty     m0_low_h                ; Scroll the players and missiles.
    dec     p0_low_h
    dec     p1_low_h
    dec     p2_low_h
    dec     p3_low_h
    ldy     current_scroll          ; Test the scroll to see how many
    cpy     #$a                     ; pointers to update this frame.
    bmi     .short_frame
    ldx     #5                      ; Update 5 pointers.
    jmp     .update_pointers
.short_frame
    ldx     #4                      ; Update 4 pointers.
.update_pointers
    lda     sDLISTH                 ; Get the current_display list.
    eor     #$c                     ; Flip the 4 and 8 bits.
    sta     $ff
    mva     #$e1,$fe                ; The base pointer address.
    ldy     scroll_list_offset      ; Get the offset to start from.
.update_loop
    lda     ($fe),y                 ; Get the low byte.
    clc                             ; The list being updated is 4 bytes
    adc     #8                      ; behind, so add 8 to get the new location.
    bcc     .no_carry
    sta     $fd
    iny
    lda     ($fe),y
    adc     #0                      ; Carry bit is set, this increments.
    sta     ($fe),y
    dey
    lda     $fd
.no_carry
    sta     ($fe),y
    iny
    iny
    iny                             ; Add 3 to y to reach the next pointer.
    dex
    bne     .update_loop
    ldx     current_scroll          ; Retest the current scroll to see if
    beq     .flip_lists             ; the display lists need to flip.
    sty     scroll_list_offset      ; No flip.  Update the offset and
    dex                             ; adjust the fine scroll.
    stx     current_scroll
    stx     HSCROL
    rts
.flip_lists
    mvy     #$f,current_scroll      ; Reset the fine scroll.
    sty     HSCROL
    mvy     #0,scroll_list_offset   ; Reset the offset.
    lda     sDLISTH                 ; Flip the high byte of the display list.
    eor     #$c                     ; Toggle the 4 and 8 bits.
    sta     sDLISTH
    sta     DLISTH
    rts

; Erase the shot cursor used during aim mode.
erase_aim_cursor subroutine
    mva     aim_cursor,$fe          ; Ready the cursor address.
    mva     aim_cursor + 1,$ff
    ldx     #4                      ; 5 pixels high.
    ldy     #0
.loop
    tya
    sta     ($fe),y                 ; Blank the location.
    lda     $fe                     ; Go to the next row.
    clc
    adc     #bytes_per_row
    bcc     .no_carry
    inc     $ff
.no_carry
    sta     $fe
    dex
    bpl     .loop
    rts

; Draw the curl gauge using the pattern at $fd.
reset_curl_gauge subroutine
    ldy     #0
    ldx     #24                     ; 24 pixels high.
    mwa     curl_meter_base,$fe     ; Set the base address.
.loop
    lda     $fd                     ; Get the pattern.
    sta     ($fe),y
    dex
    beq     .done
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry
    inc     $ff
.no_carry
    sta     $fe
    jmp     .loop
.done
    rts
    
; Handles the blinking of the lower status rocks.
blink_status_rock subroutine
    mva     #blink_length,blink_timer
    jsr     get_rock_number         ; Get the number of the rock to blink.
    txa
    and     #1                      ; Even rocks are player 0.
    beq     .p0_blink
    mwa     p1_status_base,$fe      ; Set the status base address.
    mva     #$aa,$fd                ; Color pattern for player 1.
    jmp     .find_rock
.p0_blink
    mwa     p0_status_base,$fe      ; Set the status base address.
    mva     #$55,$fd                ; Color pattern for player 0.
.find_rock
    txa                             ; Divide the rock number by 2
    lsr                             ; to get the individual player rock.
    tax
.loop
    cpx     #0                      ; Find the specific rock.
    beq     .blink_rock
    clc
    lda     $fe                     ; Advance two bytes.
    adc     #2
    bcc     .no_blink_carry
    inc     $ff
.no_blink_carry
    sta     $fe
    dex
    jmp     .loop
.blink_rock
    ldy     #0
    lda     ($fe),y                 ; Check for existing data.
    beq     .blink_on
    jsr     erase_status_rock
    rts
.blink_on
    jsr     draw_status_rock
    rts

; Build the ring patterns for the players at a given offset in y.
build_player_rings subroutine
    ldx     #0                      ; Index into the graphics image.
.loop1
    lda     player0_rings,x
    sta     $1400,y
    lda     player1_rings,x
    sta     $1500,y
    lda     player2_rings,x
    sta     $1600,y
    lda     player3_rings,x
    sta     $1700,y
    inx
    iny
    cpx     #30                     ; After 30 rows, reverse direction.
    bne     .loop1
    dex
.loop2
    lda     player0_rings,x         ; Now mirror the top half.
    sta     $1400,y
    lda     player1_rings,x
    sta     $1500,y
    lda     player2_rings,x
    sta     $1600,y
    lda     player3_rings,x
    sta     $1700,y
    iny
    dex
    bpl     .loop2
    rts

; Build lines using the pattern in the accumulator at the offset in y.
build_lines subroutine
    ldx     #70                     ; Sheet is 70 pixels high.
.loop
    sta     $1300,y
    iny
    dex
    bne     .loop
    rts

; Ready the next shot.
prepare_next_shot subroutine
    lda     current_rock            ; Check the current rock address.
    bne     .advance_rock           ; If it's zero, this is the first rock.
    lda     #rock_data_1            ; Point to the first rock.
    jmp     .rock_pointer_ready
.advance_rock
    jsr     get_rock_number         ; Need to erase the previous status rock.
    lda     #0                      ; Reset the horizontal progress while we're here.
    sta     rock_progress,x
    sta     rock_progress_high,x
    txa
    and     #1                      ; Even rocks are player 0.
    beq     .p0_erase
    mwa     p1_status_base,$fe      ; Set the status base address.
    jmp     .find_status_rock
.p0_erase
    mwa     p0_status_base,$fe      ; Set the status base address.
.find_status_rock
    txa                             ; Divide the rock number by 2
    lsr                             ; to get the individual status rock.
    tax
.status_loop
    cpx     #0                      ; Find the specific rock.
    beq     .erase_rock
    clc
    lda     $fe                     ; Advance two bytes.
    adc     #2
    bcc     .no_carry
    inc     $ff
.no_carry
    sta     $fe
    dex
    jmp     .status_loop
.erase_rock
    jsr     erase_status_rock
    lda     current_rock
    clc                             ; Point to the next rock.
    adc     #rock_data_size
.rock_pointer_ready
    sta     current_rock
    tay                             ; Set the rock data.
    lda     <#throw_address_right
    sta     rock_address,y
    lda     >#throw_address_right
    sta     rock_address + 1,y
    lda     #0
    sta     rock_offset,y
    lda     #6
    sta     rock_handle,y
    jsr     get_rock_number
    txa
    and     #1                      ; Even rocks are player 0.
    beq     .set_p0
    lda     #1
    sta     rock_flags,y
    jmp     .done_flags
.set_p0
    lda     #0
    sta     rock_flags,y
.done_flags
    txa
    jsr     position_next_shot
    mva     #blink_length,blink_timer
    jsr     clear_active_rocks
    mva     #130,m0_low_h           ; Position the hogline.
    mva     #255,m3_low_h           ; Put the back line offscreen.
    mva     #shot_mode_aim,shot_mode
    mwa     $2a91,aim_cursor        ; Initialize the shot mode.
    mva     #100,last_stick         ; Approximate center.
    mva     TRIG0,trigger_buffer    ; Initialize the trigger buffer.
    mva     #0,trigger_timer        ; Reset the debounce timer.
    sta     rock_curl               ; Reset the curl tracking.
    sta     rock_curl + 1
    mva     #shot_end_delay_time,shot_end_delay
    jsr     draw_curler
    jsr     reset_scroll
    rts

; Add the rock in x to the end of the active rocks list.
activate_rock subroutine
    ldy     #0
.loop
    lda     rock_list,y             ; Find the end of the list.
    beq     .found_end
    iny
    jmp     .loop
.found_end
    stx     rock_list,y             ; Put the rock in the list.
    rts

; Remove the rock in x from the active rocks list.  If x
; isn't is the list, this subroutine will do bad things.
deactivate_rock subroutine
    ldy     #0
    txa
.loop
    cmp     rock_list,y             ; Find the rock in the list.
    beq     .found_rock
    iny
    jmp     .loop
.found_rock
    cpy     #16                     ; If this is the end of the list,
    beq     .end_list               ; cap it with a 0.
    iny                             ; Back the values up by one slot.
    lda     rock_list,y
    dey
    sta     rock_list,y
    iny
    jmp     .found_rock
.end_list
    lda     #0
    sta     rock_list,y
    rts

; Clear the active rock list.
clear_active_rocks subroutine
    ldy     #15
    lda     #0
.loop
    sta     rock_list,y
    dey
    bpl     .loop
    rts

; Prepare the curler graphics.
draw_curler subroutine
    ldx     #0
    txa
    ldy     #$8e
.erase_loop
    sta     $1400,y
    sta     $1500,y
    sta     $1600,y
    sta     $1700,y
    inx
    iny
    cpx     #60
    bne     .erase_loop
    ldx     #8
    ldy     #$a0
.draw_loop
    lda     curler_legs_1,x
    sta     $1400,y
    dex
    bmi     .done
    iny
    jmp     .draw_loop
.done
    mva     #$32,p0_low_h
    rts

; Move the curler one pixel down.
move_curler_down subroutine
    ldx     #0
    ldy     #$ca
.loop
    lda     $1400,y
    sta     $1401,y
    lda     $1500,y
    sta     $1501,y
    lda     $1600,y
    sta     $1601,y
    lda     $1700,y
    sta     $1701,y
    dey
    inx
    cpx     #59
    bne     .loop
    lda     #0
    sta     $1400,y
    sta     $1500,y
    sta     $1600,y
    sta     $1700,y
    rts

; Move the curler one pixel up.
move_curler_up subroutine
    ldx     #0
    ldy     #$8e
.loop
    lda     $1401,y
    sta     $1400,y
    lda     $1501,y
    sta     $1500,y
    lda     $1601,y
    sta     $1600,y
    lda     $1701,y
    sta     $1700,y
    iny
    inx
    cpx     #59
    bne     .loop
    lda     #0
    sta     $1400,y
    sta     $1500,y
    sta     $1600,y
    sta     $1700,y
    rts

; Deferred vblank handler.
game_vblank subroutine
    mva     #0,ATRACT               ; Kill the attract timer.
    sta     VDSLST                  ; Reset the hblank interrupt.
    mva     #$a0,VDSLST + 1
    mva     #ring_base,HPOSP0       ; Position the players for the top rings.
    mva     #ring_base + 8,HPOSP1
    mva     #ring_base + 16,HPOSP2
    mva     #ring_base + 24,HPOSP3
    mva     #front_hog_line,HPOSM0  ; Position missile 0 for the top hog line.
    mva     #end_cap_line,HPOSM1    ; Position missile 1 for the score capper.
    mva     #back_line,HPOSM3       ; Position missile 3 for the top back line.
    mva     #4,PRIOR                ; Playfield priority over all.

    jmp     $fcb2

; 132 feet
; 330 pixels
; 83 bytes per row
; 5810 bytes total

; $1000 top text
; $1028 score text top (end numbers)
; $1050 first score line
; $1078 team 1 scores
; $10a0 second score line
; $10c8 team 2 scores
; $10f0 third score line
; $3700 bottom display

; 70 pixels from shot start to hogline

