; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2017 Average Software.

; Various interrupt handlers

; First hblank interrupt on the game screen.  Changes the
; colors to display the upper ice view.
    org     $a000
    pha
    txa
    pha

    lda     #ice_color
    ldx     #rock_color_1
    sta     WSYNC
    sta     COLBK
    stx     COLPF1
    mva     #$19,VDSLST             ; Set the next interrupt.

    pla
    tax
    pla
    rti

; Second interrupt on the game screen.  Changes the
; player 1 and 2 colors to red for the inner ring.
    org     $a019
    pha

    mva     #$2c,VDSLST             ; Set the next interrupt.
    lda     #inner_ring_color
    sta     WSYNC
    sta     COLPM1
    sta     COLPM2

    pla
    rti

; Third interrupt on the game screen.  Changes the
; background color to black for the center line.
    org     $a02c
    pha

    mva     #$3c,VDSLST             ; Set the next interrupt.
    lda     #0                      ; Change to black.
    sta     WSYNC
    sta     COLBK

    pla
    rti

; Fourth interrupt on the game screen.  Changes the
; background color back to the ice color.
    org     $a03c
    pha

    mva     #$59,VDSLST             ; Set the next interrupt.
    lda     #ice_color
    sta     WSYNC
    sta     COLBK

    pla
    rti

; Fifth interrupt on the game screen.  Changes the
; player 1 and 2 colors back to blue for the outer ring.
    org     $a059
    pha

    mva     #$7c,VDSLST             ; Set the next interrupt.
    lda     #outer_ring_color
    sta     WSYNC
    sta     COLPM1
    sta     COLPM2

    pla
    rti

; Fourth interrupt on the game screen.  Changes back to
; the text colors for the scoreboard.  Also change
; player 1 to white for the end-cap missile.
    org     $a07c
    pha
    txa
    pha

    lda     #0                      ; Background to black.
    ldx     #$0f                    ; P1 and PF1 to white.
    sta     WSYNC
    sta     COLBK
    stx     COLPM1
    stx     COLPF1
    mva     #$9c,VDSLST             ; Set the next interrupt.

    pla
    tax
    pla
    rti

; Fifth interrupt on the game screen.  Position the players
; and missiles for the bottom sheet.
    org     $a09c
    pha

    mva     #$c8,VDSLST             ; Set the next interrupt.
    mva     m0_low_h,HPOSM0
    mva     m3_low_h,HPOSM3
    mva     p0_low_h,HPOSP0
    mva     p1_low_h,HPOSP1
    mva     p2_low_h,HPOSP2
    mva     p3_low_h,HPOSP3

    pla
    rti

; Sixth interrupt on the game screen.  Changes back to
; the ice colors for the bottom sheet.  Also sets
; the first of the player/missile colors.
    org     $a0c8
    pha
    txa
    pha

    lda     #ice_color
    ldx     #rock_color_1
    sta     WSYNC
    sta     COLBK
    stx     COLPF1
    mva     pm1_low_color_1,COLPM1
    mva     pm2_low_color_1,COLPM2
    mva     pm3_low_color,COLPM3
    mva     #$f9,VDSLST             ; Set the next interrupt.

    pla
    tax
    pla
    rti

; Seventh interrupt on the game screen.  Changes
; PM1 to its second color.
    org     $a0f9
    pha

    mva     #$1c,VDSLST             ; Set the next interrupt.
    inc     VDSLST + 1
    lda     pm1_low_color_2
    sta     COLPM1

    pla
    rti

; Eighth interrupt on the game screen.  Changes
; PM2 to its second color.
    org     $a11c
    pha

    mva     #$2d,VDSLST             ; Set the next interrupt.
    lda     pm2_low_color_2
    sta     COLPM2

    pla
    rti

; Ninth interrupt on the game screen.  Changes the background
; color to black for the center line.
    org     $a12d
    pha

    lda     #0                      ; Change to black.
    sta     WSYNC
    sta     COLBK
    mva     #$3d,VDSLST             ; Set the next interrupt.

    pla
    rti

; Tenth interrupt on the game screen.  Changes the background
; color back to the ice color.
    org     $a13d
    pha

    lda     #ice_color
    sta     WSYNC
    sta     COLBK
    mva     #$4d,VDSLST             ; Set the next interrupt.

    pla
    rti

; Eleventh interrupt on the game screen.  Changes PM2
; to its third and final color.
    org     $a14d
    pha

    lda     pm2_low_color_3
    sta     COLPM2
    mva     #$5e,VDSLST             ; Set the next interrupt.

    pla
    rti

; Twelfth interrupt on the game screen. Changes PM1
; to its third and final color.
    org     $a15e
    pha

    lda     pm1_low_color_3
    sta     COLPM1
    mva     #$6f,VDSLST             ; Set the next interrupt.

    pla
    rti

; Thirteenth interrupt on the game screen.  Changes the background
; to black for the bottom rock display.
    org     $a16f
    pha

    lda     #0
    sta     WSYNC
    sta     COLBK
    mva     #$7f,VDSLST             ; Set the next interrupt.

    pla
    rti

; Fourteenth interrupt on the game screen.  Change the colors
; of the status rocks to grey.
    org     $a17f
    pha

    lda     #$0a
    sta     WSYNC
    sta     COLPF0
    sta     COLPF1
    mva     #$92,VDSLST             ; Set the next interrupt.

    pla
    rti

; Last interrupt on the game screen.  Further shading
; of the status rocks.
    org     $a192
    pha

    lda     #$06
    sta     WSYNC
    sta     COLPF0
    sta     COLPF1

    pla
    rti

