; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2017 Average Software.

; Miscellaneous rock routines

; Draw a rock.  The byte with top-left corner of the rock
; should be at $fe.  The pixel offset should be in y,
; with the team identifier in x.  The handle position
; should be passed in the accumulator.
draw_rock subroutine
    sty     $fd                     ; Save the offset.
    sta     $fc                     ; Save the handle orientation.
    cpx     #1                      ; Get the rock color.
    beq     .team1_rock
    lda     #$54                    ; Start with three pixels of team 1 color.
    jmp     .create_top_row
.team1_rock
    lda     #$a8                    ; Start with three pixels of team 0 color.
.create_top_row
    tsx                             ; Save the current stack pointer.
    stx     $fa                     ; It's used if we don't need the second byte.
    sta     $fb                     ; Save the color pattern for later.
    ldx     $fc                     ; Check the handle orientation.
    bne     .no_0_handle            ; Position 0 is straight up.
    lda     #$30                    ; One black pixel.
    jmp     .shift_top
.no_0_handle
    and     #$30                    ; Trim down to one pixel for the top row.
.shift_top
    pha                             ; Save this row for the possible second byte.
.shift_top_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_top
    lsr
    lsr
    jmp     .shift_top_loop
.done_shift_top
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry1
    inc     $ff
.no_carry1
    sta     $fe
    lda     $fb                     ; Get the color pattern back.
    cpx     #0                      ; Check the handle orientation.
    bne     .no_0_handle_2
    ora     #$30                    ; Position 0, blacken the center pixel.
    jmp     .shift_row_2
.no_0_handle_2
    cpx     #1
    bne     .no_1_handle
    ora     #$0c                    ; Position 1, blacken the right pixel.
    jmp     .shift_row_2
.no_1_handle
    cpx     #7
    bne     .shift_row_2
    ora     #$c0                    ; Position 7, blacken the left pixel.
.shift_row_2
    ldy     $fd                     ; Get the offset.
    pha                             ; Save this for byte 2.
.shift_row_2_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_row_2
    lsr
    lsr
    jmp     .shift_row_2_loop
.done_shift_row_2
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry2
    inc     $ff
.no_carry2
    sta     $fe
    lda     $fb                     ; Get the color pattern back.
    cpx     #2                      ; Check the handle orientation.
    bne     .no_2_handle
    ora     #$3c                    ; Position 2, blacken the right and center pixels.
    jmp     .shift_row_3
.no_2_handle
    cpx     #6
    bne     .handle_center
    ora     #$f0                    ; Position 6, blacken the left and center pixels.
    jmp     .shift_row_3
.handle_center
    ora     #$30                    ; Center pixel is always black.
.shift_row_3
    ldy     $fd                     ; Get the offset.
    pha                             ; Save this for byte 2.
.shift_row_3_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_row_3
    lsr
    lsr
    jmp     .shift_row_3_loop
.done_shift_row_3
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry3
    inc     $ff
.no_carry3
    sta     $fe
    lda     $fb                     ; Get the color pattern back.
    cpx     #3                      ; Check the handle orientation.
    bne     .no_3_handle
    ora     #$0c                    ; Position 3, blacken the right pixel.
    jmp     .shift_row_4
.no_3_handle
    cpx     #4
    bne     .no_4_handle
    ora     #$30                    ; Position 4, blacken the center pixel.
    jmp     .shift_row_4
.no_4_handle
    cpx     #5
    bne     .shift_row_4
    ora     #$c0                    ; Position 5, blacken the left pixel.
.shift_row_4
    ldy     $fd                     ; Get the offset.
    pha                             ; Save this for byte 2.
.shift_row_4_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_row_4
    lsr
    lsr
    jmp     .shift_row_4_loop
.done_shift_row_4
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry4
    inc     $ff
.no_carry4
    sta     $fe
    lda     $fb                     ; Get the color pattern back.
    cpx     #4                      ; Check the handle orientation.
    bne     .no_4_handle2
    lda     #$30                    ; One black pixel for position 4.
    jmp     .shift_bottom
.no_4_handle2
    and     #$30                    ; Trim down to one pixel.
.shift_bottom
    ldy     $fd                     ; Get the offset.
    pha                             ; Save this for byte 2.
.shift_bottom_loop
    dey                             ; Shift right until y goes negative.
    bmi     .done_shift_bottom
    lsr
    lsr
    jmp     .shift_bottom_loop
.done_shift_bottom
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing byte.
    sta     ($fe),y                 ; Update the playfield.
    ldy     $fd                     ; Get the offset back.
    cpy     #2                      ; An offset less than 2 doesn't stride a byte.
    bpl     .do_second_byte
    ldx     $fa                     ; Restore the stack pointer to knock
    txs                             ; the five rows off the stack.
    rts                             ; All done.
.do_second_byte
    inc     $fe                     ; Move over one byte.
    bne     .no_overflow
    inc     $ff
.no_overflow
    lda     #4                      ; 4 - offset = bytes to shift.
    sec
    sbc     $fd
    sta     $fd
    ldx     #5                      ; Draw 5 bytes.
.byte_loop
    pla                             ; Get the byte to draw off the stack.
    ldy     $fd                     ; Get the offset.
.offset_loop
    dey                             ; Shift left until y goes negative.
    bmi     .done_shift
    asl
    asl
    jmp     .offset_loop
.done_shift
    iny                             ; Set y back to 0.
    ora     ($fe),y                 ; OR with the existing bytes.
    sta     ($fe),y                 ; Update the playfield.
    dex                             ; Quit after 5 rows.
    beq     .done_byte_2
    lda     $fe                     ; Update the address.
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow
    dec     $ff
.no_borrow
    sta     $fe
    jmp     .byte_loop              ; Do it again!
.done_byte_2
    rts

; Erase a rock.  Arguments are as draw_rock,
; excluding the team and handle position.
erase_rock subroutine
    sty     $fd                     ; Save the offset.
    lda     #$cf                    ; Load the clearing pattern for the first row.
.shift_top_loop
    dey                             ; Rotate right until y goes negative.
    bmi     .done_shift_top
    sec
    ror
    sec
    ror
    jmp     .shift_top_loop
.done_shift_top
    pha                             ; Put this row on the stack.
    sta     $fc                     ; Save it for the last row.
    lda     #$03                    ; Load the clearing pattern for the middle rows.
    ldy     $fd                     ; Get the offset back.
.shift_middle_loop
    dey                             ; Rotate right until y goes negative.
    bmi     .done_middle_shift
    sec
    ror
    sec
    ror
    jmp     .shift_middle_loop
.done_middle_shift
    pha                             ; Push three copies of this.
    pha
    pha
    lda     $fc                     ; Get the "small" row back.
    iny                             ; Set y back to 0.
    ldx     #5                      ; Draw 5 rows.
.write_loop1
    and     ($fe),y                 ; AND with the existing bytes
    sta     ($fe),y                 ; and write back to the playfield.
    dex
    beq     .done_write1            ; Done when x hits 0.
    lda     $fe                     ; Update the address.
    clc
    adc     #bytes_per_row
    bcc     .no_carry
    inc     $ff
.no_carry
    sta     $fe
    pla                             ; Get the next row off the stack.
    jmp     .write_loop1            ; Go back and do it again.
.done_write1
    ldy     $fd                     ; Get the offset back.
    cpy     #2                      ; An offset less than 2
    bpl     .second_byte            ; requires no further action.
    rts
.second_byte
    lda     #4                      ; 4 - offset = offset for byte 2.
    sec
    sbc     $fd
    tay                             ; Put the offset in y.
    sty     $fd                     ; Save it for later.
    lda     #$cf                    ; Load the clearing pattern for the first row.
.shift_top_loop2
    dey                             ; Rotate left until y goes negative.
    bmi     .done_shift_top2
    sec
    rol
    sec
    rol
    jmp     .shift_top_loop2
.done_shift_top2
    pha                             ; Put this row on the stack.
    sta     $fc                     ; Save it for the last row.
    lda     #$03                    ; Load the clearing pattern for the middle rows.
    ldy     $fd                     ; Get the offset back.
.shift_middle_loop2
    dey                             ; Rotate left until y goes negative.
    bmi     .done_middle_shift2
    sec
    rol
    sec
    rol
    jmp     .shift_middle_loop2
.done_middle_shift2
    pha                             ; Push three copies of this.
    pha
    pha
    lda     $fc                     ; Get the "small" row back.
    inc     $fe                     ; Move one byte to the right.
    bne     .no_increment
    inc     $ff
.no_increment
    iny                             ; Set y back to 0.
    ldx     #5                      ; Draw 5 rows.
.write_loop2
    and     ($fe),y                 ; AND with the existing bytes
    sta     ($fe),y                 ; and write back to the playfield.
    dex
    beq     .done_write2            ; Done when x hits 0.
    lda     $fe                     ; Update the address.
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow
    dec     $ff
.no_borrow
    sta     $fe
    pla                             ; Get the next row off the stack.
    jmp     .write_loop2            ; Go back and do it again.
.done_write2
    rts

; Draw a rock in the lower status area.  The base address of
; the rock should be in $fe, the graphics pattern should be in $fd.
draw_status_rock subroutine
    ldy     #0                      ; Index 0.
    ldx     #0                      ; Graphics index 0.
.loop
    lda     display_rock,x          ; Get the byte.
    and     $fd                     ; Mask it with the pattern.
    sta     ($fe),y                 ; Write it.
    iny
    inx
    lda     display_rock,x          ; Get the right byte.
    and     $fd                     ; Mask it.
    sta     ($fe),y                 ; Write it.
    cpx     #11                     ; Quit after 12 bytes.
    beq     .done
    jsr     advance_address
    inx
    jmp     .loop
.done
    rts

; Remove a rock in the lower status area.  The base address
; of the rock should be in $fe.
erase_status_rock subroutine
    ldy     #0                      ; Index 0.
    ldx     #5                      ; 6 rows to erase.
.loop
    lda     #0                      ; Blank out the bytes.
    sta     ($fe),y
    iny
    sta     ($fe),y
    dex
    bmi     .done
    jsr     advance_address
    jmp     .loop
.done
    rts

; Routine for advancing the address in $fe.  Tailored to
; draw_status_rock and erase_status_rock.
advance_address subroutine
    lda     $fe                     ; Get the address.
    clc
    adc     #40                     ; Go to the next row.
    bcc     .no_carry
    inc     $ff
.no_carry
    sta     $fe
    dey                             ; Reset the index.
    rts

; Draws 8 status rocks starting at the address in $fe,
; using the color pattern in $fd.
draw_8_status_rocks subroutine
    mva     $fe,$fb                 ; Save the address.
    mva     $ff,$fc
    lda     #8                      ; Store a counter, draw_status_rock
    sta     $fa                     ; clobbers all the registers.
.loop
    jsr     draw_status_rock        ; Draw the rock.
    dec     $fa
    beq     .done                   ; Quit after 8.
    lda     $fb                     ; Update the address.
    clc
    adc     #2
    bcc     .no_carry
    inc     $fc
.no_carry
    sta     $fb
    sta     $fe
    mva     $fc,$ff
    jmp     .loop                   ; Do it again!
.done
    rts

; Positions the next rock in the shooting location.
; The number of the rock should be passed in the
; accumulator.
position_next_shot subroutine
    pha                             ; Save the rock number.
    and     #1                      ; See if the rock is odd or even.
    beq     .even_rock
    ldx     #1                      ; An odd numbered rock is team 1.
    jmp     .draw_rock
.even_rock
    ldx     #0                      ; An even numbered rock is team 0.
.draw_rock
    mwa     throw_address_right,$fe ; Set the graphics address.
    ldy     #0                      ; Pixel offset 0.
    lda     #6                      ; Handle position 6.
    jsr     draw_rock               ; Put the rock in front of the hack.
    pla
    rts

; Handle all logic and interactions for the rock
; whose address is in x.
process_rock subroutine
    txa                             ; Save x for later use.
    pha
    lda     rock_address,x          ; Prepare the rock for erasure.
    sta     $fe
    lda     rock_address + 1,x
    sta     $ff
    ldy     rock_offset,x
    jsr     erase_rock
    pla
    tax
    lda     rock_flags,x            ; Check the movement mode.
    and     #rock_mode
    bne     .rock_mode_1            ; Rock mode 0 has the horizontal as primary
    jsr     move_rock_horizontal    ; Check for horizontal movement.
    lda     rock_h_speed,x          ; If the rock speed is equal to
    cmp     rock_h_time,x           ; the time, the rock just moved,
    bne     .done_move              ; so check for vertical movement.
    jsr     move_rock_vertical
    jmp     .done_move
.rock_mode_1
    jsr     move_rock_vertical      ; Check for vertical movement.
    lda     rock_v_speed,x          ; If the rock speed is equal to
    cmp     rock_v_time,x           ; the time, the rock just moved,
    bne     .done_move              ; so check for horizontal movement.
    jsr     move_rock_horizontal
.done_move
    lda     rock_address + 1,x      ; Check the upper sheet boundary.
    cmp     #$20
    bpl     .check_upper_low_byte   ; Less than $20, rock is off the top.
    jmp     .kill_rock
.check_upper_low_byte
    bne     .check_lower            ; Equal to $20, check the low byte.
    lda     rock_address,x
    cmp     #$1d
    bcs     .check_lower            ; Less than $1d, rock is off the top.
    jmp     .kill_rock
.check_lower
    lda     rock_address + 1,x      ; Check the lower sheet boundary.
    cmp     #$36
    bmi     .check_lower_low_byte   ; Greater than $35, rock is off the bottom.
    jmp     .kill_rock
.check_lower_low_byte
    cmp     #$35
    bne     .check_right
    lda     rock_address,x
    cmp     #$83
    bcc     .check_right            ; Greater than $82, rock is off the bottom.
    jmp     .kill_rock
.check_right
    jsr     get_rock_number_from_x
    lda     rock_progress_high,y    ; See how far the rock has gone.
    beq     .check_foul
    lda     rock_progress,y
    cmp     #60
    bcc     .check_foul
    jmp     .kill_rock
.check_foul
    lda     shot_mode               ; If the shot mode is not "none," check
    beq     .bounds_ok              ; the hogline for a shooting foul.
    lda     rock_progress,y
    cmp     #hogline_distance
    bcc     .bounds_ok
    txa                             ; Save the rock pointer.
    pha
    mva     #shot_mode_none,shot_mode
    sta     $fd
    jsr     reset_curl_gauge
    pla                             ; Retrieve the rock pointer.
    tax
    jmp     .kill_rock
.bounds_ok
    ldy     rock_r_time,x           ; Check for rotation.
    beq     .done_rotation          ; Timer at 0, not rotating.
    dec     rock_r_time,x           ; Drop the timer.
    bne     .done_rotation          ; Not yet at 0, keep waiting.
    ldy     rock_r_speed,x          ; Reset the timer.
    sty     rock_r_time,x
    lda     rock_flags,x            ; Check the rotation direction.
    and     #rock_rotation
    beq     .counterclockwise       ; 0 is counterclockwise.
    lda     rock_handle,x
    cmp     #7                      ; Wrap back to 0 after 7.
    beq     .wrap_high
    inc     rock_handle,x
    jmp     .done_rotation
.wrap_high
    lda     #0
    sta     rock_handle,x
    jmp     .done_rotation
.counterclockwise
    lda     rock_handle,x
    beq     .wrap_low               ; Wrap back to 7 after 0.
    dec     rock_handle,x
    jmp     .done_rotation
.wrap_low
    lda     #7
    sta     rock_handle,x
.done_rotation
    txa                             ; Save x for later user.
    pha
    lda     rock_address,x          ; Redraw the rock at its new location.
    sta     $fe
    lda     rock_address + 1,x
    sta     $ff
    lda     rock_flags,x            ; Dump the team to another location
    and     #rock_team              ; so we can put it in x.
    sta     $fd
    ldy     rock_offset,x
    lda     rock_handle,x
    ldx     $fd
    jsr     draw_rock
    pla                             ; Get the rock pointer back.
    tax
    jsr     get_rock_number_from_x  ; Get the rock number to index the friction.
    lda     rock_friction,y
    cmp     #240
    beq     .slow_down_rock
    clc
    adc     #1
    sta     rock_friction,y
    rts
.slow_down_rock
    lda     #0
    sta     rock_friction,y         ; Reset the friction counter.
    lda     rock_flags,x            ; Figure out the primary movement.
    and     #rock_mode
    beq     .horizontal_friction
    lda     rock_v_speed,x
    jmp     .check_friction
.horizontal_friction
    lda     rock_h_speed,x
.check_friction
    cmp     #1                      ; Speed 1 is too fast for
    beq     .done                   ; friction to apply.
    cmp     #19                     ; At speed 19, it's time to stop.
    bne     .apply_friction
.kill_rock
    lda     #0
    sta     rock_h_speed,x          ; Stop the rock.
    sta     rock_h_time,x
    sta     rock_v_speed,x
    sta     rock_v_time,x
    sta     rock_r_speed,x
    sta     rock_r_time,x
    jsr     deactivate_rock
    rts
.apply_friction
    clc
    adc     #1
    pha                             ; Save the value then figure out
    lda     rock_flags,x            ; where to put it.
    and     #rock_mode
    beq     .store_h_friction
    pla
    sta     rock_v_speed,x
    rts
.store_h_friction
    pla
    sta     rock_h_speed,x
.done
    rts

; Subroutine of process_rock.  Processes the horizontal
; movement timer and moves the rock accordingly.
move_rock_horizontal subroutine
    ldy     rock_h_time,x           ; Check for horizontal movement.
    beq     .done                   ; On 0, there is no movement.
    bpl     .check_move_right       ; Positive is moving right.
    inc     rock_h_time,x           ; Negative is moving left.
    bne     .done                   ; When 0 is reached, move left.
    lda     rock_h_speed,x          ; Reset the timer.
    sta     rock_h_time,x
    ldy     rock_offset,x           ; Adjust the offset.
    beq     .adjust_byte_left       ; If the offset is 0, the base shifts.
    dey
    jmp     .finish_move_left
.adjust_byte_left
    dec     rock_address,x          ; Drop the address one byte.
    bcc     .no_left_underflow      ; No need to adjust the second byte.
    dec     rock_address + 1,x
.no_left_underflow
    ldy     #3                      ; Load the new offset.
.finish_move_left
    sty     rock_offset,x           ; Update the offset.
    rts
.check_move_right
    dec     rock_h_time,x
    bne     .done                   ; When 0 is reached, move right.
    lda     rock_h_speed,x          ; Reset the timer.
    sta     rock_h_time,x
    ldy     rock_offset,x           ; Adjust the offset.
    cpy     #3
    beq     .adjust_byte_right      ; On 3, the base shifts.
    iny
    jmp     .finish_move_right
.adjust_byte_right
    inc     rock_address,x          ; Move the address up one byte.
    bne     .no_right_overflow      ; No need to adjust the second byte.
    inc     rock_address + 1,x
.no_right_overflow
    ldy     #0                      ; Load the new offset.
.finish_move_right
    sty     rock_offset,x           ; Update the offset.
.done
    rts

; Subroutine of process_rock.  Processes the vertical
; movement timer and moves the rock accordingly.
move_rock_vertical subroutine
    ldy     rock_v_time,x           ; Check the vertical movement.
    beq     .done                   ; On 0, there is no movement.
    bpl     .check_move_down        ; Positive is moving down.
    inc     rock_v_time,x           ; Negative is moving up.
    bne     .done                   ; When 0 is reached, move up.
    lda     rock_v_speed,x          ; Reset the timer.
    sta     rock_v_time,x
    lda     rock_address,x          ; Lower the address one row.
    sec
    sbc     #bytes_per_row
    bcs     .no_borrow
    dec     rock_address + 1,x
.no_borrow
    sta     rock_address,x
    lda     shot_mode               ; If the rock is being held,
    beq     .done                   ; move the curler.
    txa
    pha
    jsr     move_curler_up
    pla
    tax
    rts
.check_move_down
    dec     rock_v_time,x
    bne     .done                   ; When 0 is reached, move down.
    lda     rock_v_speed,x          ; Reset the timer.
    sta     rock_v_time,x
    lda     rock_address,x          ; Raise the address one row.
    clc
    adc     #bytes_per_row
    bcc     .no_carry
    inc     rock_address + 1,x
.no_carry
    sta     rock_address,x
    lda     shot_mode               ; If the rock is being held,
    beq     .done                   ; move the curler.
    txa
    pha
    jsr     move_curler_down
    pla
    tax
.done
    rts

; Do curl calculations on the current rock.  Assumes the
; current rock pointer is in x and the current h_speed is
; in the accumulator, and not 0.
apply_curl subroutine
    pha                             ; Save the h_speed.
    lda     rock_r_speed,x          ; Make sure the rock is spinning.
    bne     .curl_ok
    jmp     .done
.curl_ok
    lda     rock_h_speed,x          ; Check the curl progress.
    clc
    adc     rock_r_speed,x
    adc     rock_curl
    bcs     .curl_carry
    sta     rock_curl
    jmp     .done
.curl_carry
    sta     rock_curl
    inc     rock_curl + 1
    lda     rock_curl + 1
    cmp     #128
    bcc     .done
.perform_curl
    mva     #0,rock_curl + 1
    lda     rock_flags,x            ; Figure out the direction of spin.
    and     #rock_rotation      
    beq     .counterclockwise       ; 0 is counter clockwise.
    ldy     rock_v_speed,x          ; Figure out the vertical direction.
    bmi     .cw_up
    beq     .cw_neutral
    cpy     #1                      ; Speed 1 is as fast as we get.
    beq     .done
    dec     rock_v_speed,x          ; Speed up the rock.
    jmp     .done
.cw_up
    cpy     #-8                     ; -8 is the low boundary.
    beq     .cw_up_reverse
    dec     rock_v_speed,x          ; Slow down the rock.
    jmp     .done
.cw_up_reverse
    ldy     #0
    sty     rock_v_speed,x          ; Go to neutral.
    sty     rock_v_time,x
    jmp     .done
.cw_neutral
    ldy     #8
    sty     rock_v_speed,x          ; Slowest speed down.
    sty     rock_v_time,x
    jmp     .done
.counterclockwise
    ldy     rock_v_speed,x          ; Figure out the vertical direction.
    bmi     .ccw_up
    beq     .ccw_neutral
    cpy     #8                      ; 8 is the low boundary.
    beq     .ccw_down_reverse
    inc     rock_v_speed,x          ; Slow down the rock.
    jmp     .done
.ccw_down_reverse
    ldy     #0
    sty     rock_v_speed,x          ; Go to neutral.
    sty     rock_v_time,x
    jmp     .done
.ccw_up
    cpy     #-1                     ; Speed -1 is as fast as we get.
    beq     .done
    inc     rock_v_speed,x          ; Speed up the rock.
    jmp     .done
.ccw_neutral
    ldy     #-10
    sty     rock_v_speed,x          ; Slowest speed up.
    sty     rock_v_time,x
.done
    pla                             ; Retrieve the h_speed.
    rts

