; This file is part of RealSports Curling.

; RealSports Curling is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.

; RealSports Curling is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; You should have received a copy of the GNU General Public License
; along with RealSports Curling.  If not, see <http://www.gnu.org/licenses/>.

; Copyright 2017 Average Software.

; Miscellaneous startup routines

; One-time initialization upon power up.
init_system subroutine
    jsr     clear_zero_page         ; Clear RAM and hardware.
    jsr     clear_ram
    jsr     clear_hardware
    jsr     init_common_vectors     ; Set system vectors.
    jsr     init_common_registers   ; Set system registers.
    rts

; Blank the zero page, requires slightly different handling.
clear_zero_page subroutine
    lda     #0
    tay                             ; Set the index to 0.
.loop
    sta     $0,y
    iny
    bne     .loop                   ; Jump back if the page isn't done.
    rts

; Blank the page of RAM in $ff.
clear_page subroutine
    mva     #0,$fe                  ; Set the low byte to 0.
    tay                             ; Set the index to 0.
.loop
    sta     ($fe),y
    iny
    bne     .loop                   ; Jump back if the page isn't done.
    rts

; Clear all memory from $200 to $3fff.
clear_ram subroutine
    ldx     #2                      ; Start on page 2.
.loop
    stx     $ff
    jsr     clear_page
    inx
    cpx     #$40                    ; Quit upon reaching $4000.
    bne     .loop
    rts

; Blanks the memory pages for GTIA, ANTIC, and POKEY.
clear_hardware subroutine
    mva     #$c0,$ff                ; Clear GTIA.
    jsr     clear_page
    mva     #$d4,$ff                ; Clear ANTIC.
    jsr     clear_page
    mva     #$e8,$ff                ; Clear POKEY.
    jsr     clear_page
    rts

; Initialize the system vectors that are common to all modes.
init_common_vectors subroutine
    mwa     $fc03,VIMIRQ            ; Set the main IRQ vector.
    mwa     $fcb8,VVBLKI            ; Set the VBI vector.
    mwa     $fd02,VKYBDI            ; Set the keyboard IRQ vector.
    mwa     $fcb2,VKYBDF            ; Set the deferred keyboard vector.
    mwa     $fcb2,VVBLKD            ; Set the deferred VBI vector.
    rts

; Initialize the various registers that are common to all modes.
init_common_registers subroutine
    mva     #$40,sIRQEN             ; Enable keyboard interrupts.
    sta     IRQEN             
    mva     #$3e,sDMACTL            ; Enable DMA.
    sta     DMACTL            
    mva     #$f8,CHBASE             ; Set the character base.
    mva     #$10,PMBASE             ; Set the player/missile address.
    mva     #$03,GRACTL             ; Enable player/missiles.
    mva     #4,CONSOL               ; Enable POT scanning.
    mva     #2,SKCTL                ; Enable keypad scanning.
    rts

